@@ -22,6 +22,8 @@
 #ifndef CHECK_H
 #define CHECK_H
 
+#include <stddef.h>
+
 /* Check: a unit test framework for C
 
    Check is a unit test framework for C. It features a simple
@@ -59,17 +61,21 @@
 CK_CPPSTART
 #endif
 
+#include <sys/types.h>
 
 /* check version numbers */
 
 #define CHECK_MAJOR_VERSION (0)
 #define CHECK_MINOR_VERSION (9)
-#define CHECK_MICRO_VERSION (2)
+#define CHECK_MICRO_VERSION (3)
 
 extern int check_major_version;
 extern int check_minor_version;
 extern int check_micro_version;
 
+#ifndef NULL
+#define NULL ((void*)0)
+#endif
 
 /* opaque type for a test case
 
@@ -161,14 +167,20 @@ static void __testname (void)\
 #define END_TEST }
 
 /* Fail the test case unless expr is true */
+/* The space before the comma sign before ## is essential to be compatible
+   with gcc 2.95.3 and earlier.
+*/
 #define fail_unless(expr, ...)\
         _fail_unless(expr, __FILE__, __LINE__,\
         "Assertion '"#expr"' failed" , ## __VA_ARGS__, NULL)
 
 /* Fail the test case if expr is true */
+/* The space before the comma sign before ## is essential to be compatible
+   with gcc 2.95.3 and earlier.
+*/
 #define fail_if(expr, ...)\
         _fail_unless(!(expr), __FILE__, __LINE__,\
-        "Failure '"#expr"' occured", ## __VA_ARGS__, NULL)
+        "Failure '"#expr"' occured" , ## __VA_ARGS__, NULL)
 
 /* Always fail */
 #define fail(...) _fail_unless(0, __FILE__, __LINE__, "Failed" , ## __VA_ARGS__, NULL)
@@ -320,6 +332,12 @@ enum fork_status srunner_fork_status (SRunner *sr);
 /* Set the current fork status */
 void srunner_set_fork_status (SRunner *sr, enum fork_status fstat); 
   
+/* Fork in a test and make sure messaging and tests work. */
+pid_t check_fork(void);
+
+/* Wait for the pid and exit. If pid is zero, just exit. */
+void check_waitpid_and_exit(pid_t pid);
+
 #ifdef __cplusplus 
 #define CK_CPPEND }
 CK_CPPEND
