@@ -36,6 +36,7 @@
 typedef struct TF {
   TFun fn;
   const char *name;
+  int signal;
 } TF;
 
 struct Suite {
@@ -51,6 +52,7 @@ typedef struct Fixture
 
 struct TCase {
   const char *name;
+  int timeout;
   List *tflst; /* list of test functions */
   List *unch_sflst;
   List *unch_tflst;
@@ -70,6 +72,7 @@ struct TestResult {
   char *file;    /* File where the test occured */
   int line;      /* Line number where the test occurred */
   const char *tcname;  /* Test case that generated the result */
+  const char *tname;  /* Test that generated the result */
   char *msg;     /* Failure message */
 };
 
@@ -77,6 +80,8 @@ TestResult *tr_create(void);
 void tr_reset(TestResult *tr);
 
 enum cl_event {
+  CLINITLOG_SR,
+  CLENDLOG_SR,
   CLSTART_SR,
   CLSTART_S,
   CLEND_SR,
@@ -99,6 +104,7 @@ struct SRunner {
   TestStats *stats; /* Run statistics */
   List *resultlst; /* List of unit test results */
   const char *log_fname; /* name of log file */
+  const char *xml_fname; /* name of xml output file */
   List *loglst; /* list of Log objects */
   enum fork_status fstat; /* controls if suites are forked or not
 			     NOTE: Don't use this value directly,
