@@ -80,11 +80,20 @@ static int waserror (int status, int expected_signal);
 
 #define MSG_LEN 100
 
-static pid_t current_test;
+static int alarm_received;
+static pid_t group_pid;
 
 static void sig_handler(int sig_nr)
 {
-  kill(current_test, SIGKILL);
+  switch (sig_nr) {
+   case SIGALRM:
+    alarm_received = 1;
+    killpg(group_pid, SIGKILL);
+    break;
+   default:
+    eprintf("Unhandled signal: %d", __FILE__, __LINE__, sig_nr);
+    break;
+  }
 }
 
 static void srunner_run_init (SRunner *sr, enum print_output print_mode)
@@ -199,7 +208,7 @@ static int srunner_run_unchecked_setup (SRunner *sr, TCase *tc)
   l = tc->unch_sflst;
 
   for (list_front(l); !list_at_end(l); list_advance(l)) {
-    send_ctx_info(get_send_key(),CK_CTX_SETUP);
+    send_ctx_info(CK_CTX_SETUP);
     f = list_val(l);
     f->fun();
 
@@ -228,12 +237,12 @@ static TestResult * tcase_run_checked_setup (SRunner *sr, TCase *tc)
   
   l = tc->ch_sflst;
   if (fstat == CK_FORK) {
-    send_ctx_info(get_send_key(),CK_CTX_SETUP);
+    send_ctx_info(CK_CTX_SETUP);
   }
   
   for (list_front(l); !list_at_end(l); list_advance(l)) {
     if (fstat == CK_NOFORK) {
-      send_ctx_info(get_send_key(),CK_CTX_SETUP);
+      send_ctx_info(CK_CTX_SETUP);
     }
     f = list_val(l);
     f->fun();
@@ -262,7 +271,7 @@ static void tcase_run_checked_teardown (TCase *tc)
 
   l = tc->ch_tflst;
   
-  send_ctx_info(get_send_key(),CK_CTX_TEARDOWN);
+  send_ctx_info(CK_CTX_TEARDOWN);
 
   for (list_front(l); !list_at_end(l); list_advance(l)) {
     f = list_val(l);
@@ -281,7 +290,7 @@ static void srunner_run_unchecked_teardown (SRunner *sr, TCase *tc)
   for (list_front(l); !list_at_end(l); list_advance(l)) {
     
     f = list_val(l);
-    send_ctx_info(get_send_key(),CK_CTX_TEARDOWN);
+    send_ctx_info(CK_CTX_TEARDOWN);
     f->fun ();
   }
   set_fork_status(srunner_fork_status(sr));
@@ -301,8 +310,7 @@ static TestResult *receive_result_info_fork (const char *tcname,
 {
   TestResult *tr;
 
-  tr = receive_test_result(get_recv_key(),
-                           waserror(status, expected_signal));
+  tr = receive_test_result(waserror(status, expected_signal));
   if (tr == NULL)
     eprintf("Failed to receive test result", __FILE__, __LINE__);
   tr->tcname = tcname;
@@ -317,7 +325,7 @@ static TestResult *receive_result_info_nofork (const char *tcname,
 {
   TestResult *tr;
 
-  tr = receive_test_result(get_recv_key(), 0);
+  tr = receive_test_result(0);
   if (tr == NULL)
     eprintf("Failed to receive test result", __FILE__, __LINE__);
   tr->tcname = tcname;
@@ -336,12 +344,20 @@ static void set_fork_info (TestResult *tr, int status, int signal_expected)
 
   if (was_sig) {
     if (signal_expected == signal_received) {
-      tr->rtype = CK_PASS;
-      tr->msg = pass_msg();
-    } else if (signal_expected != 0) { /* signal received, but no the expected one */
+      if (alarm_received) {
+        /* Got alarm instead of signal */
+        tr->rtype = CK_ERROR;
+        tr->msg = signal_error_msg(signal_received, signal_expected);
+      } else {
+        tr->rtype = CK_PASS;
+        tr->msg = pass_msg();
+      }
+    } else if (signal_expected != 0) {
+      /* signal received, but not the expected one */
       tr->rtype = CK_ERROR;
       tr->msg = signal_error_msg(signal_received, signal_expected);
-    } else {                           /* signal received and none expected */
+    } else {
+      /* signal received and none expected */
       tr->rtype = CK_ERROR;
       tr->msg = signal_msg(signal_received);
     }
@@ -395,6 +411,7 @@ static TestResult *tcase_run_tfun_nofork (SRunner *sr, TCase *tc, TF *tfun)
   
 static TestResult *tcase_run_tfun_fork (SRunner *sr, TCase *tc, TF *tfun)
 {
+  pid_t pid_w;
   pid_t pid;
   int status = 0;
 
@@ -402,17 +419,23 @@ static TestResult *tcase_run_tfun_fork (SRunner *sr, TCase *tc, TF *tfun)
   if (pid == -1)
      eprintf("Unable to fork:",__FILE__,__LINE__);
   if (pid == 0) {
+    setpgid(0, 0);
+    group_pid = getpgid(0);
     tcase_run_checked_setup(sr, tc);
     tfun->fn();
     tcase_run_checked_teardown(tc);
     exit(EXIT_SUCCESS);
+  } else {
+    group_pid = getpgid(pid);
   }
 
-  current_test = pid;
+  alarm_received = 0;
   alarm(tc->timeout);
   do {
-    pid = wait(&status);
-  } while (pid == -1);
+    pid_w = waitpid(pid, &status, 0);
+  } while (pid_w == -1);
+  
+  killpg(pid, SIGKILL); /* Kill remaining processes. */
 
   return receive_result_info_fork(tc->name, tfun->name, status, tfun->signal);
 }
@@ -424,8 +447,13 @@ static char *signal_error_msg (int signal_received, int signal_expected)
   char *msg = emalloc (MSG_LEN); /* free'd by caller */
   sig_r_str = strdup(strsignal(signal_received));
   sig_e_str = strdup(strsignal(signal_expected));
-  snprintf (msg, MSG_LEN, "Error: Received signal %d (%s), expected %d (%s)",
-            signal_received, sig_r_str, signal_expected, sig_e_str);
+  if (alarm_received) {
+    snprintf (msg, MSG_LEN, "Test timeout expired, expected signal %d (%s)",
+              signal_expected, sig_e_str);
+  } else {
+    snprintf (msg, MSG_LEN, "Received signal %d (%s), expected %d (%s)",
+              signal_received, sig_r_str, signal_expected, sig_e_str);
+  }
   free(sig_r_str);
   free(sig_e_str);
   return msg;
@@ -433,9 +461,13 @@ static char *signal_error_msg (int signal_received, int signal_expected)
 
 static char *signal_msg (int signal)
 {
-  char *msg = emalloc (MSG_LEN); /* free'd by caller */
-  snprintf (msg, MSG_LEN, "Received signal %d (%s)",
-            signal, strsignal(signal));
+  char *msg = emalloc(MSG_LEN); /* free'd by caller */
+  if (alarm_received) {
+    snprintf(msg, MSG_LEN, "Test timeout expired");
+  } else {
+    snprintf(msg, MSG_LEN, "Received signal %d (%s)",
+             signal, strsignal(signal));
+  }
   return msg;
 }
 
@@ -473,6 +505,29 @@ void srunner_set_fork_status (SRunner *sr, enum fork_status fstat)
   sr->fstat = fstat;
 }
 
+pid_t check_fork (void)
+{
+  pid_t pid = fork();
+  /* Set the process to a process group to be able to kill it easily. */
+  setpgid(pid, group_pid);
+  return pid;
+}
+
+void check_waitpid_and_exit (pid_t pid)
+{
+  pid_t pid_w;
+  int status;
+
+  if (pid > 0) {
+    do {
+      pid_w = waitpid(pid, &status, 0);
+    } while (pid_w == -1);
+    if (waserror(status, 0))
+      exit(EXIT_FAILURE);
+  }
+  exit(EXIT_SUCCESS);
+}  
+
 static int waserror (int status, int signal_expected)
 {
   int was_sig = WIFSIGNALED (status);
