@@ -1,11 +1,13 @@
 #include <stdlib.h>
 #include <signal.h>
 #include <stdio.h>
+#include <string.h>
 #include <check.h>
 #include "check_error.h"
 #include "check_str.h"
 #include "check_check.h"
-#include "check_magic.h"
+
+static char errm[200];
 
 static void fixture_sub_setup (void)
 {
@@ -13,11 +15,12 @@ static void fixture_sub_setup (void)
 }
 
 
-static Suite *fixture_s;
 static SRunner *fixture_sr;
+
 void setup_fixture (void)
 {
   TCase *tc;
+  Suite *fixture_s;
   
   fixture_s = suite_create("Fix Sub");
   tc = tcase_create("Core");
@@ -30,7 +33,6 @@ void setup_fixture (void)
 void teardown_fixture (void)
 {
   srunner_free(fixture_sr);
-  suite_free(fixture_s);
 }
 
 START_TEST(test_fixture_fail_counts)
@@ -48,7 +50,7 @@ END_TEST
 START_TEST(test_print_counts)
 {
   char *srstat = sr_stat_str(fixture_sr);
-  char *exp = "0%: Checks: 0, Failures: 1, Errors: 0";
+  const char *exp = "0%: Checks: 0, Failures: 1, Errors: 0";
 
   fail_unless(strcmp(srstat, exp) == 0,
 	      "SRunner stat string incorrect with setup failure");
@@ -59,15 +61,13 @@ START_TEST(test_setup_failure_msg)
 {
   TestResult **tra;
   char *trm;
-  char *trmexp = "check_check_fixture.c:12:S:Core: Test failure in fixture";
+  const char *trmexp = "check_check_fixture.c:14:S:Core:unchecked_setup: Test failure in fixture";
 
   tra = srunner_failures(fixture_sr);
   trm = tr_str(tra[0]);
 
-  if (strcmp(trm, trmexp) != 0) {
-    char *errm = emalloc(CK_MAXMSG);
-
-    snprintf(errm,CK_MAXMSG,
+  if (strstr(trm, trmexp) == 0) {
+    snprintf(errm, sizeof(errm),
 	     "Bad setup tr msg (%s)", trm);
     
     fail (errm);
@@ -117,7 +117,7 @@ START_TEST(test_ch_setup)
   fail_unless(srunner_ntests_failed(sr) == 0,
 	      "Checked setup not being run correctly");
 
-  suite_free(s);
+
   srunner_free(sr);
 }
 END_TEST
@@ -144,9 +144,15 @@ static void teardown_sub_signal(void)
   raise(SIGFPE);
 }
 
+START_TEST(test_sub_fail)
+{
+  fail("Should never run");
+}
+END_TEST
+
 START_TEST(test_sub_pass)
 {
-  ; /*pass*/
+  fail_unless(1 == 1, "Always pass");
 }
 END_TEST
 
@@ -161,15 +167,15 @@ START_TEST(test_ch_setup_fail)
   s = suite_create("Setup Fail");
   tc = tcase_create("Core");
   suite_add_tcase(s, tc);
-  tcase_add_test(tc,test_sub_pass);
+  tcase_add_test(tc,test_sub_fail);
   tcase_add_checked_fixture(tc,setup_sub_fail, NULL);
   sr = srunner_create(s);
   srunner_run_all(sr,CK_SILENT);
 
-  fail_unless (srunner_ntests_failed(sr) == 1,
-	       "Failure counts not correct for checked setup failure");
   fail_unless (srunner_ntests_run(sr) == 0,
 	       "Test run counts not correct for checked setup failure");
+  fail_unless (srunner_ntests_failed(sr) == 1,
+	       "Failure counts not correct for checked setup failure");
 
   strstat= sr_stat_str(sr);
 
@@ -180,12 +186,10 @@ START_TEST(test_ch_setup_fail)
 
   trm = tr_str(srunner_failures(sr)[0]);
 
-  if (strcmp(trm,
-	     "check_check_fixture.c:127:S:Core: Failed setup")
-      != 0) {
-    char *errm = emalloc(CK_MAXMSG);
-    
-    snprintf(errm,CK_MAXMSG,
+  if (strstr(trm,
+	     "check_check_fixture.c:127:S:Core:test_sub_fail: Failed setup")
+      == 0) {
+    snprintf(errm, sizeof(errm),
 	     "Bad failed checked setup tr msg (%s)", trm);
     
     fail (errm);
@@ -193,6 +197,75 @@ START_TEST(test_ch_setup_fail)
 }
 END_TEST
 
+START_TEST(test_ch_setup_fail_nofork)
+{
+  TCase *tc;
+  Suite *s;
+  SRunner *sr;
+
+  s = suite_create("Setup Fail");
+  tc = tcase_create("Core");
+  suite_add_tcase(s, tc);
+  tcase_add_test(tc, test_sub_fail);
+  tcase_add_checked_fixture(tc, setup_sub_fail, NULL);
+  sr = srunner_create(s);
+  srunner_set_fork_status(sr, CK_NOFORK);
+  srunner_run_all(sr, CK_SILENT);
+
+  fail_unless (srunner_ntests_run(sr) == 0,
+	       "Test run counts not correct for checked setup failure");
+  fail_unless (srunner_ntests_failed(sr) == 1,
+	       "Failure counts not correct for checked setup failure");
+}
+END_TEST
+
+START_TEST(test_ch_setup_fail_nofork_2)
+{
+  TCase *tc;
+  Suite *s;
+  SRunner *sr;
+
+  s = suite_create("Setup Fail");
+  tc = tcase_create("Core");
+  suite_add_tcase(s, tc);
+  tcase_add_test(tc, test_sub_fail);
+  tcase_add_checked_fixture(tc, sub_ch_setup_norm, NULL);
+  tcase_add_checked_fixture(tc, setup_sub_fail, NULL);
+  sr = srunner_create(s);
+  srunner_set_fork_status(sr, CK_NOFORK);
+  srunner_run_all(sr, CK_SILENT);
+
+  fail_unless (srunner_ntests_run(sr) == 0,
+	       "Test run counts not correct for checked setup failure");
+  fail_unless (srunner_ntests_failed(sr) == 1,
+	       "Failure counts not correct for checked setup failure");
+}
+END_TEST
+
+START_TEST(test_ch_setup_pass_nofork)
+{
+  TCase *tc;
+  Suite *s;
+  SRunner *sr;
+
+  s = suite_create("Setup Fail");
+  tc = tcase_create("Core");
+  suite_add_tcase(s, tc);
+  tcase_add_test(tc, test_sub_pass);
+  tcase_add_checked_fixture(tc, sub_ch_setup_norm, NULL);
+  tcase_add_checked_fixture(tc, sub_ch_setup_norm, NULL);
+  tcase_add_checked_fixture(tc, sub_ch_setup_norm, NULL);
+  sr = srunner_create(s);
+  srunner_set_fork_status(sr, CK_NOFORK);
+  srunner_run_all(sr, CK_SILENT);
+
+  fail_unless (srunner_ntests_run(sr) == 1,
+	       "Test run counts not correct for checked setup failure");
+  fail_unless (srunner_ntests_failed(sr) == 0,
+	       "Failure counts not correct for checked setup failure");
+}
+END_TEST
+
 START_TEST(test_ch_setup_sig)
 {
   TCase *tc;
@@ -204,7 +277,7 @@ START_TEST(test_ch_setup_sig)
   s = suite_create("Setup Fail");
   tc = tcase_create("Core");
   suite_add_tcase(s, tc);
-  tcase_add_test(tc,test_sub_pass);
+  tcase_add_test(tc,test_sub_fail);
   tcase_add_checked_fixture(tc,setup_sub_signal, NULL);
   sr = srunner_create(s);
   srunner_run_all(sr,CK_SILENT);
@@ -223,13 +296,11 @@ START_TEST(test_ch_setup_sig)
 
   trm = tr_str(srunner_failures(sr)[0]);
 
-  if (strcmp(trm,
-	     "check_check_fixture.c:137:S:Core: "
+  if (strstr(trm,
+	     "check_check_fixture.c:137:S:Core:test_sub_fail: "
 	     "(after this point) Received signal 8")
-      != 0) {
-    char *errm = emalloc(CK_MAXMSG);
-    
-    snprintf(errm,CK_MAXMSG,
+      == 0) {
+    snprintf(errm, sizeof(errm),
 	     "Msg was (%s)", trm);
     
     fail (errm);
@@ -255,24 +326,22 @@ START_TEST(test_ch_teardown_fail)
 
   fail_unless (srunner_ntests_failed(sr) == 1,
 	       "Failure counts not correct for checked teardown failure");
-  fail_unless (srunner_ntests_run(sr) == 0,
+  fail_unless (srunner_ntests_run(sr) == 1,
 	       "Test run counts not correct for checked teardown failure");
 
   strstat= sr_stat_str(sr);
 
   fail_unless(strcmp(strstat,
-		     "0%: Checks: 0, Failures: 1, Errors: 0") == 0,
+		     "0%: Checks: 1, Failures: 1, Errors: 0") == 0,
 	      "SRunner stat string incorrect with checked setup failure");
 
 
   trm = tr_str(srunner_failures(sr)[0]);
 
-  if (strcmp(trm,
-	     "check_check_fixture.c:132:S:Core: Failed teardown")
-      != 0) {
-    char *errm = emalloc(CK_MAXMSG);
-    
-    snprintf(errm,CK_MAXMSG,
+  if (strstr(trm,
+	     "check_check_fixture.c:132:S:Core:test_sub_pass: Failed teardown")
+      == 0) {
+    snprintf(errm, sizeof(errm),
 	     "Bad failed checked teardown tr msg (%s)", trm);
     
     fail (errm);
@@ -299,25 +368,23 @@ START_TEST(test_ch_teardown_sig)
 
   fail_unless (srunner_ntests_failed(sr) == 1,
 	       "Failure counts not correct for checked teardown signal");
-  fail_unless (srunner_ntests_run(sr) == 0,
+  fail_unless (srunner_ntests_run(sr) == 1,
 	       "Test run counts not correct for checked teardown signal");
 
   strstat= sr_stat_str(sr);
 
   fail_unless(strcmp(strstat,
-		     "0%: Checks: 0, Failures: 0, Errors: 1") == 0,
+		     "0%: Checks: 1, Failures: 0, Errors: 1") == 0,
 	      "SRunner stat string incorrect with checked teardown signal");
 
 
   trm = tr_str(srunner_failures(sr)[0]);
 
-  if (strcmp(trm,
-	     "check_check_fixture.c:143:S:Core: "
+  if (strstr(trm,
+	     "check_check_fixture.c:143:S:Core:test_sub_pass: "
 	     "(after this point) Received signal 8")
-      != 0) {
-    char *errm = emalloc(CK_MAXMSG);
-    
-    snprintf(errm,CK_MAXMSG,
+      == 0) {
+    snprintf(errm, sizeof(errm),
 	     "Bad msg (%s)", trm);
     
     fail (errm);
@@ -341,6 +408,9 @@ Suite *make_fixture_suite (void)
   tcase_add_test(tc,test_setup_failure_msg);
   tcase_add_test(tc,test_ch_setup);
   tcase_add_test(tc,test_ch_setup_fail);
+  tcase_add_test(tc,test_ch_setup_fail_nofork);
+  tcase_add_test(tc,test_ch_setup_fail_nofork_2);
+  tcase_add_test(tc,test_ch_setup_pass_nofork);
   tcase_add_test(tc,test_ch_setup_sig);
   tcase_add_test(tc,test_ch_teardown_fail);
   tcase_add_test(tc,test_ch_teardown_sig);
