@@ -1,21 +1,28 @@
 #include <stdlib.h>
 #include <signal.h>
 #include <unistd.h>
+#include <sys/types.h>
+#include <sys/wait.h>
 #include <check.h>
 #include "check_check.h"
 
 
 
+
+
+
+
+
 START_TEST(test_lno)
 {
-  fail("Failure expected"); /*line 11*/
+  fail("Failure expected"); /*line 18*/
 }
 END_TEST
 
 START_TEST(test_mark_lno)
 {
-  mark_point(); /*line 17*/
-  exit(1); /*should fail at line 17*/
+  mark_point(); /*line 24*/
+  exit(EXIT_FAILURE); /*should fail at line 24*/
 }
 END_TEST
 
@@ -101,7 +108,7 @@ START_TEST(test_fail_empty)
 }
 END_TEST
 
-START_TEST(test_segv) /* line 104 */
+START_TEST(test_segv) /* line 111 */
 {
   raise (SIGSEGV);
 }
@@ -129,7 +136,7 @@ START_TEST(test_mark_point)
 }
 END_TEST
 
-START_TEST(test_eternal) /* line 132 */
+START_TEST(test_eternal) /* line 139 */
 {
   for (;;)
     ;
@@ -142,13 +149,13 @@ START_TEST(test_sleep2)
 }
 END_TEST
 
-START_TEST(test_sleep5) /* line 145 */
+START_TEST(test_sleep5) /* line 152 */
 {
   sleep(5);
 }
 END_TEST
 
-START_TEST(test_sleep8) /* line 151 */
+START_TEST(test_sleep8) /* line 158 */
 {
   sleep(8);
 }
@@ -157,7 +164,7 @@ END_TEST
 
 START_TEST(test_early_exit)
 {
-  exit(1);
+  exit(EXIT_FAILURE);
 }
 END_TEST
 
@@ -187,6 +194,110 @@ START_TEST(test_null_2)
 }
 END_TEST
 
+START_TEST(test_fork1p_pass)
+{
+  pid_t pid;
+  
+  if((pid = fork()) < 0) {
+    fail("Failed to fork new process");
+  } else if (pid > 0) {
+    fail_unless(1);
+    kill(pid, SIGKILL);
+  } else {
+    for (;;) {
+      sleep(1);
+    }
+  }
+}
+END_TEST
+
+START_TEST(test_fork1p_fail)
+{
+  pid_t pid;
+  
+  if((pid = fork()) < 0) {
+    fail("Failed to fork new process");
+  } else if (pid > 0) {
+    fail("Expected fail");
+    kill(pid, SIGKILL);
+  } else {
+    for (;;) {
+      sleep(1);
+    }
+  }
+}
+END_TEST
+
+START_TEST(test_fork1c_pass)
+{
+  pid_t pid;
+  
+  if((pid = check_fork()) < 0) {
+    fail("Failed to fork new process");
+  } else if (pid > 0) {
+    check_waitpid_and_exit(pid);
+  } else {
+    fail_unless(1);
+    check_waitpid_and_exit(0);
+  }
+}
+END_TEST
+
+START_TEST(test_fork1c_fail)
+{
+  pid_t pid;
+  
+  if((pid = check_fork()) < 0) {
+    fail("Failed to fork new process");
+  } else if (pid == 0) {
+    fail("Expected fail");
+    check_waitpid_and_exit(0);
+  }
+  check_waitpid_and_exit(pid);
+}
+END_TEST
+
+START_TEST(test_fork2_pass)
+{
+  pid_t pid;
+  pid_t pid2;
+  
+  if((pid = check_fork()) < 0) {
+    fail("Failed to fork new process");
+  } else if (pid > 0) {
+    if((pid2 = check_fork()) < 0) {
+      fail("Failed to fork new process");
+    } else if (pid2 == 0) {
+      fail_unless(1);
+      check_waitpid_and_exit(0);
+    }
+    check_waitpid_and_exit(pid2);
+  }
+  check_waitpid_and_exit(pid);
+}
+END_TEST
+
+START_TEST(test_fork2_fail)
+{
+  pid_t pid;
+  pid_t pid2;
+  
+  if((pid = check_fork()) < 0) {
+    fail("Failed to fork new process");
+  } else if (pid > 0) {
+    if((pid2 = check_fork()) < 0) {
+      fail("Failed to fork new process");
+    } else if (pid2 == 0) {
+      fail("Expected fail");
+      check_waitpid_and_exit(0);
+    }
+    check_waitpid_and_exit(pid2);
+    fail("Expected fail");
+  }
+  check_waitpid_and_exit(pid);
+}
+END_TEST
+
 START_TEST(test_2nd_suite)
 {
   fail("We failed");
@@ -206,11 +317,14 @@ Suite *make_sub2_suite(void)
 Suite *make_sub_suite(void)
 {
   Suite *s = suite_create("Check Servant");
+
   TCase *tc_simple = tcase_create("Simple Tests");
   TCase *tc_signal = tcase_create("Signal Tests");
   TCase *tc_timeout = tcase_create("Timeout Tests");
   TCase *tc_timeout_ext = tcase_create("Extended Timeout Tests");
   TCase *tc_limit = tcase_create("Limit Tests");
+  TCase *tc_messaging_and_fork = tcase_create("Msg and fork Tests");
+
   suite_add_tcase (s, tc_simple);
   suite_add_tcase (s, tc_signal);
   suite_add_tcase (s, tc_timeout);
@@ -218,6 +332,8 @@ Suite *make_sub_suite(void)
   /* Add a second time to make sure tcase_set_timeout doesn't contaminate it. */
   suite_add_tcase (s, tc_timeout);
   suite_add_tcase (s, tc_limit);
+  suite_add_tcase (s, tc_messaging_and_fork);
+
   tcase_add_test (tc_simple, test_lno);
   tcase_add_test (tc_simple, test_mark_lno);
   tcase_add_test (tc_simple, test_pass);
@@ -232,6 +348,7 @@ Suite *make_sub_suite(void)
   tcase_add_test (tc_simple, test_fail_vararg_msg_2);
   tcase_add_test (tc_simple, test_fail_vararg_msg_3);
   tcase_add_test (tc_simple, test_fail_empty);
+
   tcase_add_test (tc_signal, test_segv);
   tcase_add_test_raise_signal (tc_signal, test_segv, 11); /* pass  */
   tcase_add_test_raise_signal (tc_signal, test_segv, 8);  /* error */
@@ -239,17 +356,28 @@ Suite *make_sub_suite(void)
   tcase_add_test_raise_signal (tc_signal, test_fail, 8);  /* fail  */
   tcase_add_test (tc_signal, test_fpe);
   tcase_add_test (tc_signal, test_mark_point);
+
   tcase_add_test (tc_timeout, test_eternal);
   tcase_add_test (tc_timeout, test_sleep2);
   tcase_add_test (tc_timeout, test_sleep5);
   tcase_add_test (tc_timeout, test_sleep8);
+
   tcase_set_timeout (tc_timeout_ext, 6);
   tcase_add_test (tc_timeout_ext, test_eternal);
   tcase_add_test (tc_timeout_ext, test_sleep2);
   tcase_add_test (tc_timeout_ext, test_sleep5);
   tcase_add_test (tc_timeout_ext, test_sleep8);
+
   tcase_add_test (tc_limit, test_early_exit);
   tcase_add_test (tc_limit, test_null);
   tcase_add_test (tc_limit, test_null_2);
+
+  tcase_add_test (tc_messaging_and_fork, test_fork1p_pass);
+  tcase_add_test (tc_messaging_and_fork, test_fork1p_fail);
+  tcase_add_test (tc_messaging_and_fork, test_fork1c_pass);
+  tcase_add_test (tc_messaging_and_fork, test_fork1c_fail);
+  tcase_add_test (tc_messaging_and_fork, test_fork2_pass);
+  tcase_add_test (tc_messaging_and_fork, test_fork2_fail);
+
   return s;
 }
