@@ -1,11 +1,17 @@
 #!/bin/sh
 
+if [ "${srcdir}" = "." ]; then
+    lsrc=""
+else
+    lsrc="${srcdir}/"
+fi
+
 expected="Running suite S1
-ex_log_output.c:7:P:Core: Passed
-ex_log_output.c:13:F:Core: Failure
-ex_log_output.c:17:E:Core: (after this point) Early exit with return value 1
+${lsrc}ex_log_output.c:8:P:Core:test_pass: Passed
+${lsrc}ex_log_output.c:14:F:Core:test_fail: Failure
+${lsrc}ex_log_output.c:18:E:Core:test_exit: (after this point) Early exit with return value 1
 Running suite S2
-ex_log_output.c:25:P:Core: Passed
+${lsrc}ex_log_output.c:26:P:Core:test_pass2: Passed
 Results for all suites run:
 50%: Checks: 4, Failures: 1, Errors: 1"
 
